"""Stub file for the '_bisect' module."""

from typing import Any, Sequence, TypeVar

_T = TypeVar('_T')
def bisect(a: Sequence[_T], x: _T, lo: int = ..., hi: int = ...) -> int: ...
def bisect_left(a: Sequence[_T], x: _T, lo: int = ..., hi: int = ...) -> int: ...
def bisect_right(a: Sequence[_T], x: _T, lo: int = ..., hi: int = ...) -> int: ...
def insort(a: Sequence[_T], x: _T, lo: int = ..., hi: int = ...) -> None: ...
def insort_left(a: Sequence[_T], x: _T, lo: int = ..., hi: int = ...) -> None: ...
def insort_right(a: Sequence[_T], x: _T, lo: int = ..., hi: int = ...) -> None: ...
