from typing import Any

class _TempModule:
    mod_name = ...  # type: Any
    module = ...  # type: Any
    def __init__(self, mod_name): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...

class _ModifiedArgv0:
    value = ...  # type: Any
    def __init__(self, value): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...

def run_module(mod_name, init_globals=None, run_name=None, alter_sys=False): ...
def run_path(path_name, init_globals=None, run_name=None): ...
